#pragma once
#include "B4RDefines.h"
#include <WiFi.h>
#include <WiFiClientSecure.h>
#include <rom/rtc.h>

extern "C" {
//#include "user_interface.h"
}

//~version: 1.20

namespace B4R {
	
	//~shortname: ESP32Pins
	//Pins constants for ESP32 boards.
	class ESP32_Pins {
		public:
		static const Byte LEDpin		= 2;
		static const Byte BUTTONpin	= 0;
		static const Byte RX0pin		= 34; 
		static const Byte TX0pin		= 35; 
		static const Byte RX1pin		= 9; 
		static const Byte TX1pin		= 10; 
		static const Byte RX2pin		= 16; 
		static const Byte TX2pin		= 17; 
		static const Byte SDApin		= 21; 
		static const Byte SCLpin		= 22; 
		static const Byte SPI_MOSIpin	= 23;
		static const Byte SPI_MISOpin	= 19;
		static const Byte SPI_CLKpin	= 18;
		static const Byte SPI_SSpin	= 5;
		};
	
	//~shortname: ESP32extras
	class B4RESP32extras {
		public:
			void Restart();
			void DeepSleepTime(int Secs);
			void DeepSleepPin(int Pin, int Level);
			int ResetReason();
			static const int REASON_POWERON_RESET			= 1;
			static const int REASON_SW_RESET				= 3;
			static const int REASON_OWDT_RESET				= 4;
			static const int REASON_DEEPSLEEP_RESET			= 5;
			static const int REASON_SDIO_RESET				= 6;
			static const int REASON_TG0WDT_SYS_RESET		= 7;
			static const int REASON_TG1WDT_SYS_RESET		= 8;
			static const int REASON_RTCWDT_SYS_RESET		= 9;
			static const int REASON_INTRUSION_RESET			= 10;
			static const int REASON_TGWDT_CPU_RESET			= 11;
			static const int REASON_SW_CPU_RESET			= 12;
			static const int REASON_RTCWDT_CPU_RESET		= 13;
			static const int REASON_EXT_CPU_RESET			= 14;
			static const int REASON_RTCWDT_BROWN_OUT_RESET	= 15;
			static const int REASON_RTCWDT_RTC_RESET		= 16;
			
			ulong FlashChipSpeed();
			ulong CpuCycleCount();
			ulong FreeHeapSpace();
			byte CpuFreqMHz();
			
			void GetMacAddress(ArrayByte* mac);
			void ConfigNetwork(ArrayByte* ip,ArrayByte* gateway,ArrayByte* subnet,ArrayByte* dns1,ArrayByte* dns2);
			bool ConfigAP(B4RString* ip, B4RString* gateway, B4RString* subnet);	
			void GetAPIP(ArrayByte* ip);
			B4RString* IP2str(ArrayByte* ip);
			ArrayByte* str2IP(B4RString* sip);
			bool Host2IP(B4RString* Host, ArrayByte* ip);
			bool IsBadIP(ArrayByte* ip);
			int GetInternetIP(ArrayByte* ip);
			bool ConnectWPS();
			bool ConnectWPS2(UInt Position);
			B4RString* SSID();
			void GetLocalIP(ArrayByte* ip);
			void GetSubnetMask(ArrayByte* ip);
			void GetGatewayIP(ArrayByte* ip);

		private:
			IPAddress b4str2IPaddress(B4RString* sip);
			IPAddress str2IPaddress(byte* sip, int L);
	};
	extern WiFiGenericClass WiFiGeneric;
}
